package com.bocloud.sms.booter.config;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * api权限白名单设置
 *
 * @author tyl
 * @since 2021/3/9 18:10
 */
@Component
public class WhiteApiCache implements InitializingBean {
    private final StringRedisTemplate redisTemplate;
    private final ThreadPoolTaskExecutor taskExecutor;

    public WhiteApiCache(StringRedisTemplate redisTemplate, ThreadPoolTaskExecutor taskExecutor) {
        this.redisTemplate = redisTemplate;
        this.taskExecutor = taskExecutor;
    }


    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(() -> {
            //设置白名单
            List<String> whiteList = Lists.newArrayList("^GET:/v1/users/permissions$",
                    "^GET:/v1/login/detail$", "^GET:/v1/trx/randomstr$", "^POST:/v1/trx/login$", "^POST:/v1/trx/logout$", "^GET:/v1/messages$", "^GET:/v1/token$","^POST:/v1/token/check$",
                    "^POST:/v1/users/logout$", "^POST:/v1/tenants/logout$", "^GET:/v1/config/license$", "^GET:/v1/roles/ids$");
            redisTemplate.opsForValue().set("permission_white", JSONObject.toJSONString(whiteList));
        });
    }
}
