package com.bocloud.sms.booter.controller;

import com.bocloud.sms.interfaces.IndexService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 首页
 *
 * @author tyl
 * @since 2021/03/25
 */
@RestController
@RequestMapping("/v1/index")
@Tag(name = "首页")
public class IndexDataController {

    @Autowired
    private IndexService indexService;


    /**
     * 待办工单列表
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/doneWorkOrder")
    @Operation(summary = "待办工单列表")
    public GeneralResult doneWorkOrder(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.doneWorkOrder(pager, requestContext);
    }

    /**
     * 工单统计
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/workOrder/stattic")
    @Operation(summary = "工单统计")
    public GeneralResult workOrderStatic( @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.workOrderStatic(requestContext);
    }

    /**
     * 当前用户首页数据展示模块的数据查询
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/userData/static")
    @Operation(summary = "当前用户首页数据展示模块的数据查询")
    public GeneralResult userStatic(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.userDataStatic(requestContext);
    }

    /**
     * 当前用户授权应用列表
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/userAppList")
    @Operation(summary = "当前用户首页数据展示模块的数据查询")
    public GeneralResult userAppList(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.userAppList(requestContext);
    }

    /**
     * 近7日告警趋势
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/sevenDayWarnTrend")
    @Operation(summary = "近7日告警趋势")
    public GeneralResult sevenDayWarnTrend(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.sevenDayWarnTrend(requestContext);
    }

    /**
     * 单日告警统计
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/oneDayWarnCount")
    @Operation(summary = "单日告警统计")
    public GeneralResult oneDayWarnCount(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.oneDayWarnCount(requestContext);
    }

    /**
     * 告警列表
     * @param pager
     * @param requestContext
     * @return
     */
    @GetMapping("/warnList")
    @Operation(summary = "告警列表")
    public GeneralResult warnList(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return indexService.warnList(requestContext);
    }
}
