package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.SystemNotice;
import com.bocloud.sms.interfaces.SystemNoticeService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 *
 */
@RestController
@RequestMapping("/v1/system/notice")
@Tag(name = "系统公告管理")
public class SystemNoticeController {

    private final SystemNoticeService service;

    public SystemNoticeController(SystemNoticeService systemNoticeService) {
        this.service = systemNoticeService;
    }


    /**
     * 创建
     *
     * @param entity
     * @param requestContext
     * @return
     */
    @Operation(summary = "创建")
    @PostMapping
    public Result create(@RequestBody SystemNotice entity, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.create(entity, requestContext);
    }

    /**
     * 删除
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Operation(summary = "删除")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.remove(id, requestContext);
    }

    /**
     * 修改
     *
     * @param id
     * @param entity
     * @param requestContext
     * @return
     */
    @Operation(summary = "修改")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody SystemNotice entity,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.modify(entity, requestContext);
    }

    /**
     * 发布
     *
     * @param id
     * @return
     */
    @Operation(summary = "发布")
    @PostMapping("/send/{id}")
    public Result send(@PathVariable(value = Common.ID) Long id) {
        return service.sendMsg(id);
    }

    /**
     * 列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Operation(summary = "列表")
    @GetMapping
    public GeneralResult<GridBean<SystemNotice>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.list(pager, requestContext);
    }

    /**
     * 详情
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Operation(summary = "详情")
    @GetMapping("/{id}")
    public GeneralResult<SystemNotice> detail(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.detail(id);
    }
}
