package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.User;
import com.bocloud.sms.entity.UserGroup;
import com.bocloud.sms.interfaces.UserGroupService;
import com.bocloud.sms.model.UserGroupBean;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author hongqingyu
 */
@RestController
@RequestMapping("/v1/users/groups")
@Tag(name = "用户管理")
public class UserGroupController {

    @Autowired
    private UserGroupService userGroupService;

    /**
     * 获取用户组列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(summary = "查询用户组列表")
    @GetMapping
    public GeneralResult<GridBean<User>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return userGroupService.list(pager);
    }

    /**
     * 创建用户组
     *
     * @param context
     * @return
     */
    @Operation(summary = "创建用户组")
    @PostMapping
    public Result create(@RequestBody UserGroup group, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return userGroupService.create(group, context);
    }

    /**
     * 修改用户组
     *
     * @param context
     * @return
     */
    @Operation(summary = "用户组修改")
    @PutMapping("/{id}")
    public Result modify(@RequestBody UserGroupBean bean, @Value(Common.REQ_CONTEXT) RequestContext context,
                         @PathVariable(value = Common.ID) Long id) {
        bean.setId(id);
        return userGroupService.modify(bean, context);
    }

    /**
     * 移除用户组
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "删除用户组")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return userGroupService.remove(id, context);
    }

    /**
     * 查看用户组详细信息
     *
     * @param id
     * @return
     */
    @Operation(summary = "用户组详情")
    @GetMapping("/{id}")
    public GeneralResult<UserGroup> detail(@PathVariable(value = Common.ID) Long id) {
        return userGroupService.detail(id);
    }

    /**
     * 配置用户与用户组的关系
     *
     * @return
     */
    @PostMapping("/relation")
    @Operation(summary = "关联账号")
    public Result relation(@RequestBody UserGroupBean bean) {
        return userGroupService.configUser(bean.getUserIds(), bean.getId());
    }

    /**
     * 通过userId查询用户组信息
     *
     * @param id 用户id
     * @return
     */
    @GetMapping("/relation/users/{id}")
    @Operation(summary = "根据用户id查询用户组")
    public Result listGroupIdByUserId(@PathVariable Long id) {
        return userGroupService.listGroupIdByUserId(id);
    }

    /**
     * 查看用户组详细信息
     *
     * @param id
     * @return
     */
    @Operation(summary = "用户组用户详情")
    @GetMapping("/{id}/users")
    public GeneralResult<UserGroup> users(Pager pager, @PathVariable(value = Common.ID) Long id) {
        return userGroupService.listUserByGroupId(pager, id);
    }

    /**
     * 获取用户组列表
     *
     * @return
     */
    @Operation(summary = "获取全量用户组")
    @GetMapping("/all")
    public GeneralResult<GridBean<User>> listAll() {
        return userGroupService.listAll();
    }
}
