package com.bocloud.sms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.booter.websocket.message.MessageHandler;
import com.bocloud.sms.booter.websocket.model.CommonMessage;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;
import io.jsonwebtoken.lang.Assert;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.stereotype.Component;

/**
 * 站内信通知监听
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年8月10日
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class BocloudMessageListener implements MessageListener {

    private final MessageHandler messageHandler;

    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (log.isDebugEnabled()) {
                log.debug("get bocloud message:{}", content);
            }
            Assert.notNull(content, "消息内容为空");
            GeneralResult<JSONObject> result = JSONTools.isJSON(content);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            OperateResult operate = JSONObject.parseObject(content, OperateResult.class);
            Assert.notNull(content, "消息格式不正确");
            RequestContext context = operate.getContext();
            if (null != context && null != context.getTarget() && 0 != context.getTarget()) {
                // 发送消息给指定ID的用户前端
                messageHandler.sendMessageToUser(context.getCatalog(), context.getTarget(), new CommonMessage(content));
            } else {// 发送给所有的用户前端
                messageHandler.sendMessageToUsers(new CommonMessage(content));
            }
        } catch (Exception e) {
            log.error("send websocket message failed: {}", e.getMessage(), e);
        }
    }
}
