package com.bocloud.sms.booter.scheduler;

import com.bocloud.sms.entity.SystemNotice;
import com.bocloud.sms.interfaces.SystemNoticeService;
import com.bocloud.sms.repository.SystemNoticeRepository;
import com.bocloud.sms.service.SystemNoticeServiceImpl;
import com.google.common.collect.Lists;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
@RequiredArgsConstructor
public class SystemNoticeScheduler implements InitializingBean {

    private final CurrentService service;
    private final SystemNoticeService systemNoticeService;
    private final SystemNoticeRepository systemNoticeRepository;
    private final LockFactory lockFactory;

    @Scheduled(cron = "0 */2 * * * ?")
    protected void sendNotice() {
        log.debug("[执行]定时发送系统公告");
        log.debug("当前服务是否是leader：{}", service.isLeader());
        if (!service.isLeader()) {
            return;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String path = SystemNoticeScheduler.class.getSimpleName() + "_sendNotice";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 获取未发送且已经到发送时间的系统公告进行发送
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("status", SystemNoticeServiceImpl.StatusEnum.NOTSEND), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("sendTime", sdf.format(new Date())), Sign.LET));
            List<SystemNotice> notices = systemNoticeRepository.list(params);
            Assert.notNull(notices, "没有数据");
            for (SystemNotice notice : notices) {
                systemNoticeService.sendMsg(notice.getId());
            }
        } catch (Exception e) {
            log.error("SystemNoticeScheduler sendNotice error:", e);
        }

    }

    @Override
    public void afterPropertiesSet() {
        this.sendNotice();
    }
}
