package com.bocloud.sms.interfaces;

import com.bocloud.sms.entity.Business;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.model.ProjectModel;
import com.bocloud.sms.model.ProjectUserBean;
import com.megatron.common.model.*;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author tyl
 * @time 2020/6/2 周二 15:57
 */
public interface ProjectService {

    /**
     * 分页查询项目列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    GeneralResult<GridBean<Project>> list(Pager pager, RequestContext requestContext, Boolean tenantId);

    /**
     * 字典查询项目信息
     *
     * @param condition
     * @param requestContext
     * @return
     */
    GeneralResult condition(String condition, RequestContext requestContext);

    /**
     * 创建项目
     *
     * @param params
     * @param requestContext
     * @return
     */
    Result create(ProjectModel params, RequestContext requestContext);

    /**
     * 修改项目
     *
     * @param id
     * @param params
     * @param requestContext
     * @return
     */
    Result modify(Long id, ProjectModel params, RequestContext requestContext);

    /**
     * 删除项目
     *
     * @param id
     * @param requestContext
     * @return
     */
    Result remove(Long id, RequestContext requestContext);

    /**
     * 批量删除项目
     *
     * @param ids
     * @param requestContext
     * @return
     */
    Result batchRemove(List<Long> ids, RequestContext requestContext);

    /**
     * 查询详细
     *
     * @param id
     * @return
     */
    GeneralResult<Project> detail(Long id);


    /**
     * 项目下配置用户
     *
     * @param projectId
     * @param projectUserBean
     * @return
     */
    Result configUser(Long projectId, ProjectUserBean projectUserBean);

    /**
     * 根据查询条件导出项目列表
     *
     * @param params
     * @param requestContext
     * @return
     */
    XSSFWorkbook export(List<Param> params, RequestContext requestContext);

    /**
     * 查询项目所在业务详细信息
     *
     * @param id
     * @param requestContext
     * @return
     */
    GeneralResult<Business> getBusinessDetail(Long id, RequestContext requestContext);

    /**
     * 设置项目成员为项目经理
     *
     * @param id
     * @param userId
     * @return
     */
    Result configProjectManager(Long id, Long userId);

    /**
     * 导入模板下载
     *
     * @param title
     * @param requestContext
     * @return
     */
    XSSFWorkbook importTemp2(String title, RequestContext requestContext);

    /**
     * 导入模板下载
     *
     * @param response
     * @return
     */
    Result importTemp(HttpServletResponse response);

    /**
     * 导入项目
     *
     * @param multipartFile
     * @param requestContext
     * @return
     */
    Result importData(MultipartFile multipartFile, RequestContext requestContext);

    GeneralResult<List<Project>> queryByDepart(String departName);

    GeneralResult<List<Project>> queryByDepartIds(Pager pager, List<Long> departIds);

    GeneralResult<Project> getByProjectName(String departName);
}
