package com.bocloud.sms.repository;

import com.bocloud.sms.entity.Admission;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @time 2020/2/4 周二 22:31
 */
@Repository("admissionRepository")
public class AdmissionRepository extends BasicGenericDao<Admission, Long> {

    static final String SELECT = "select a.*,b.name creator from admission a left join users b on b.id = a.creator_id";

    public AdmissionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 查询ip访问控制列表
     *
     * @param pager 分页查询对象
     * @return 访问控制列表
     * @throws Exception 数据库异常
     */
    public List<Admission> list(Pager pager) throws Exception {
        String sql = SELECT + " where 1 = 1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = getQueryBuilder().getParam(pager.getParams());
        return this.list(Admission.class, sql, param);
    }

    /**
     * 查询ip访问控制列表
     *
     * @param params 查询参数列表
     * @return 访问控制列表
     * @throws Exception 数据库异常
     */
    public List<Admission> list(List<Param> params) throws Exception {
        String sql = SELECT + " where 1 = 1 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, new HashMap<>(16, 0.75F)), "a");
        return this.list(Admission.class, sql);
    }

    /**
     * 获取ip访问控制数量
     *
     * @param params 查询参数列表
     * @return 访问控制列表大小
     * @throws Exception 数据库异常
     */
    public int count(List<Param> params) throws Exception {
        String sql = "select count(id) from admission where 1 = 1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 删除ID访问控制
     *
     * @param id 访问控制ID
     * @return 删除结果
     * @throws Exception 数据库异常
     */
    public boolean remove(Long id) throws Exception {
        String sql = "delete from admission where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    /**
     * 查询ip访问控制详情
     *
     * @param id 访问控制ID
     * @return 访问控制对象
     * @throws Exception 数据库异常
     */
    public Admission query(Long id) throws Exception {
        String sql = SELECT + " where a.id = :id ";
        List<Admission> list = this.list(Admission.class, sql, MapTools.simpleMap("id", id));
        Assert.isTrue(list.size() <= 1, "more than one admission in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据IP查询
     *
     * @param ip IP地址
     * @return 访问控制对象
     * @throws Exception 运行异常
     */
    public Admission getByIp(String ip) throws Exception {
        String sql = SELECT + " where a.ip = :ip";
        Map<String, Object> paramMap = MapTools.simpleMap("ip", ip);
        List<Admission> list = this.list(Admission.class, sql, paramMap);
        Assert.isTrue(list.size() <= 1, "more than one admission in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询IP访问控制列表
     *
     * @return 访问控制列表
     * @throws Exception 运行异常
     */
    public List<Admission> list() throws Exception {
        return this.list(Admission.class, SELECT);
    }

}
